import { Button, Card, Form, Input, message, Modal, Radio, Select, Space, Upload } from "antd"
import React, { useEffect, useMemo, useState } from "react"
import '../../tencentAdPutIn/index.less'
import { useAjax } from "@/Hook/useAjax"
import { getUserAllApi } from "@/services/operating/account"
import { RcFile } from "antd/lib/upload"
import { UploadOutlined } from "@ant-design/icons"
import style from './index.less'
import CropperImg from "@/components/FileBoxAD/components/Cropper"

interface Props {
    visible?: boolean
    onChange?: () => void
    onClose?: () => void
}
/**
 * 上传素材
 * @returns 
 */
const UploadFile: React.FC<Props> = ({ visible, onClose }) => {

    /**********************************/
    const [queryForm, setQueryForm] = useState<CLOUDNEW.AddMaterialProps>({ materialType: 'image' })
    const [fileList, setFileList] = useState<any>([])
    const [fileUrl, setFileUrl] = useState<string>('')
    const [previewVisible, setPreviewVisible] = useState<boolean>(false)
    const [visibleCropper, setVisibleCropper] = useState<boolean>(false)

    const getUserAll = useAjax(() => getUserAllApi())
    /**********************************/

    useEffect(() => {
        getUserAll.run()
    }, [])

    const handleOk = () => {

    }

    const getVideo = useMemo(() => {
        if (queryForm?.materialType === 'video') {
            if (queryForm?.file) {
                return <video src={URL.createObjectURL(queryForm?.file)} style={{ width: 200 }} controls />
            } else {
                return null
            }
        } else {
            return null
        }
    }, [queryForm?.file])

    return <Modal
        title={<strong>{'上传素材'}</strong>}
        open={visible}
        onCancel={onClose}
        className="modalResetCss"
        bodyStyle={{ backgroundColor: '#f1f4fc', maxHeight: 650, overflow: 'hidden', overflowY: 'auto', padding: '10px' }}
        maskClosable={false}
        width={600}
    >
        <Card className="cardResetCss">
            <Form
                labelAlign='left'
                labelCol={{ span: 4 }}
                layout="horizontal"
                colon={false}
                style={{}}
            >
                <Form.Item label={<strong>素材名称</strong>}>
                    <Input placeholder="素材名称(不填是上传素材名称)" value={queryForm?.materialName} onChange={(e) => setQueryForm({ ...queryForm, materialName: e.target.value })} />
                </Form.Item>

                <Form.Item label={<strong>设计师</strong>}>
                    <Select
                        placeholder="请选择设计师(不选默认自己)"
                        filterOption={(input, option) =>
                            ((option?.children ?? '') as any).toLowerCase().includes(input.toLowerCase())
                        }
                        value={queryForm?.designerId}
                        onChange={(e => setQueryForm({ ...queryForm, designerId: e }))}
                        options={getUserAll?.data?.map((item: { nickname: any; userId: any }) => ({ label: item.nickname, value: item.userId }))}
                    />
                </Form.Item>

                <Form.Item label={<strong>素材类型</strong>}>
                    <Radio.Group value={queryForm?.materialType} onChange={(e) => setQueryForm({ ...queryForm, materialType: e.target.value })} buttonStyle="solid">
                        <Radio.Button value="image">图片</Radio.Button>
                        <Radio.Button value="video">视频</Radio.Button>
                    </Radio.Group>
                </Form.Item>

                <Form.Item label={<strong>描述</strong>}>
                    <Input.TextArea placeholder="描述" value={queryForm?.description} onChange={(e) => setQueryForm({ ...queryForm, description: e.target.value })} />
                </Form.Item>

                <Form.Item label={<strong>{'上传图片'}</strong>}>
                    {queryForm?.materialType === 'image' ? <Space>
                        <Upload
                            listType="picture-card"
                            accept='image/gif,image/jpeg,image/png,image/jpg'
                            beforeUpload={(file: RcFile) => {
                                return false
                            }}
                            fileList={fileList}
                            onChange={(newFileList: any) => {
                                console.log('newFileList-->', newFileList)
                                setQueryForm({ ...queryForm, file: newFileList.file })
                                setFileList([...newFileList.fileList])
                            }}
                            onPreview={(file: any) => {
                                setPreviewVisible(true)
                                setFileUrl(file.thumbUrl)
                            }}
                            onRemove={() => {
                                setQueryForm({ ...queryForm, file: null })
                            }}
                        >
                            {fileList?.length < 1 && '普通上传'}
                        </Upload>
                        {
                            fileList?.length === 0 && <Upload
                                listType="picture-card"
                                accept='image/gif,image/jpeg,image/png,image/jpg'
                                beforeUpload={(file: RcFile) => {
                                    return false
                                }}
                                fileList={fileList}
                                onChange={(newFileList: any) => {
                                    setQueryForm({ ...queryForm, file: newFileList.file })
                                    setVisibleCropper(true)
                                }}
                                onRemove={() => {
                                    setQueryForm({ ...queryForm, file: null })
                                }}
                                onPreview={(file: any) => {
                                    setPreviewVisible(true)
                                    setFileUrl(file.thumbUrl)
                                }}
                            >
                                {fileList?.length < 1 && '裁剪上传'}
                            </Upload>
                        }
                    </Space> : <>
                        <div className={style.file}>
                            <Button type='primary'><UploadOutlined />上传视频</Button>
                            <input type='file' onChange={(e) => {
                                if (e?.target?.files) {
                                    setQueryForm({ ...queryForm, file: e?.target?.files[0] })
                                }
                            }} accept="video/*" />
                        </div>
                        {queryForm?.file && <>
                            <p>{queryForm?.file?.name}</p>
                            {getVideo}
                        </>}
                    </>}

                </Form.Item>
            </Form>
        </Card>

        {/* 预览 */}
        <Modal
            open={previewVisible}
            footer={null}
            onCancel={() => {
                setPreviewVisible(false)
            }}
        >
            <img alt="example" style={{ width: '100%' }} src={fileUrl} />
        </Modal>

        {/* 裁剪 */}
        {visibleCropper && <CropperImg visible={visible} onClose={() => setVisibleCropper(false)} file={queryForm.file} onChange={(fileList: any[], file: any) => { setFileList(fileList); setQueryForm({ ...queryForm, file: file }); setVisibleCropper(false) }} />}
    </Modal>
}

export default React.memo(UploadFile)