import { request } from 'umi';
import { api } from '../api';


/**
 * 获取文件夹列表
 * @param data 
 * @returns 
 */
export async function getFolderListApi(data: CLOUDNEW.GetFolderListProps) {
    return request(api + `/material/folder/listByPage`, {
        method: 'POST',
        data
    })
}

/**
 * 新建文件夹
 * @param data 
 * @returns 
 */
export async function addFolderApi(data: CLOUDNEW.AddFolderProps) {
    return request(api + `/material/folder/create`, {
        method: 'POST',
        data
    })
}

/**
 * 修改文件夹
 * @param data 
 * @returns 
 */
export async function updateFolderApi(data: CLOUDNEW.AddFolderProps) {
    return request(api + `/material/folder/update`, {
        method: 'POST',
        data
    })
}


/**
 * 删除文件夹
 * @param id 
 * @returns 
 */
export async function delFolderApi(id: number) {
    return request(api + `/material/folder/delete/${id}`, {
        method: 'DELETE'
    })
}


/**
 * 移动文件夹
 * @param param0 
 * @returns 
 */
export async function moveFolderApi({ targetId, sourceId }: { targetId: number, sourceId: number[] }) {
    return request(api + `/material/folder/move`, {
        method: 'POST',
        data: sourceId,
        params: { targetId }
    })
}

/**
 * 文件夹详情
 * @param id 
 * @returns 
 */
export async function getFolderDetailsApi(id: number) {
    return request(api + `/material/folder/detail/${id}`, {
        method: 'GET'
    })
}


/**
 * 分页查询素材（无数据）
 * @param data 
 * @returns 
 */
export async function getMaterialListApi(data: CLOUDNEW.GetMaterialListProps) {
    return request(api + `/material/material/pageList`, {
        method: 'POST',
        data
    })
}


/**
 * 新增素材
 * @param data 
 * @returns 
 */
export async function addMaterialApi(data: CLOUDNEW.AddMaterialProps) {
    return request(api + `/material/material/add`, {
        method: 'POST',
        data
    })
}


/**
 * 修改素材
 * @param data 
 * @returns 
 */
export async function updateMaterialApi(data: CLOUDNEW.UpdateMaterialProps) {
    return request(api + `/material/material/update`, {
        method: 'POST',
        data
    })
}


/**
 * 删除素材
 * @param id 
 * @returns 
 */
export async function delMaterialApi(id: number) {
    return request(api + `/material/material/delete/${id}`, {
        method: 'DELETE'
    })
}